unit Unit1;

{

Preserving Set Type Values in INI Files (/Databases)
http://zarko-gajic.iz.hr/preserving-set-type-values-in-ini-files-databases/

-arko Gaji

}

interface

uses
  System.TypInfo, System.IniFiles,
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls,
  Vcl.CheckLst;

type
  TUIElement = (uiButton = 0, uiCheckBox, uiRadioButton, uiPanel, uiGroupBox, uiEdit);
  TUIElements = set of TUIElement;

  TForm1 = class(TForm)
    Button1: TButton;
    Edit1: TEdit;
    Panel1: TPanel;
    CheckBox1: TCheckBox;
    RadioButton1: TRadioButton;
    GroupBox1: TGroupBox;
    CheckListBox1: TCheckListBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure CheckListBox1ClickCheck(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CheckListBox1Enter(Sender: TObject);
  private
    fVisibleUIElements: TUIElements;

    function VisibleElementsAll : TUIElements;

    procedure UpdateUIForVisibility;

    function VisibleUIElementsToInteger : integer;
    procedure VisibleUIElementsFromInteger(const vuiInt : integer);
  public
    property VisibleUIElements : TUIElements read fVisibleUIElements;

    procedure SaveToIni;
    procedure LoadFromIni;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.CheckListBox1ClickCheck(Sender: TObject);
var
  i : integer;
begin
  fVisibleUIElements := [];

  for i := 0 to -1 + CheckListBox1.Items.Count do
    if CheckListBox1.Checked[i] then
      fVisibleUIElements := fVisibleUIElements + [TUIElement(i)];

  UpdateUIForVisibility;
end;

procedure TForm1.CheckListBox1Enter(Sender: TObject);
begin
  CheckListBox1.ItemIndex := -1;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveToIni;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  ui : TUIElement;
  uiInt : integer;
begin
  for ui in [Low(TUIElement) .. High(TUIElement)] do
    CheckListBox1.Items.Add(Copy(GetEnumName(TypeInfo(TUIElement), integer(ui)), 3));

  LoadFromIni;

  uiInt := VisibleUIElementsToInteger;
  for ui in [Low(TUIElement) .. High(TUIElement)] do
    CheckListBox1.Checked[Integer(ui)] := 1 shl Integer(ui) AND uiInt <> 0;
end;

procedure TForm1.LoadFromIni;
var
  uiInt : integer;
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini')) do
  try
    uiInt := ReadInteger('config', 'VisibleUIElements', 0);
    VisibleUIElementsFromInteger(uiInt);
    UpdateUIForVisibility;
  finally
    Free;
  end;
end;

procedure TForm1.SaveToIni;
var
  uiInt : integer;
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini')) do
  try
    WriteInteger('config', 'VisibleUIElements', VisibleUIElementsToInteger);
  finally
    Free;
  end;
end;

procedure TForm1.UpdateUIForVisibility;
begin
  Button1.Visible := uiButton in VisibleUIElements;
  Edit1.Visible := uiEdit in VisibleUIElements;
  Panel1.Visible := uiPanel in VisibleUIElements;
  GroupBox1.Visible := uiGroupBox in VisibleUIElements;
  CheckBox1.Visible := uiCheckBox in VisibleUIElements;
  RadioButton1.Visible := uiRadioButton in VisibleUIElements;
end;

function TForm1.VisibleElementsAll: TUIElements;
var
  ui : TUIElement;
begin
  for ui in [Low(TUIElement) .. High(TUIElement)] do Include(result, ui);
end;

procedure TForm1.VisibleUIElementsFromInteger(const vuiInt : integer);
var
  ui : TUIElement;
begin
  fVisibleUIElements := [];
  for ui in [Low(TUIElement) .. High(TUIElement)] do
    if 1 shl Integer(ui) AND vuiInt <> 0 then
      Include(fVisibleUIElements, ui);
end;

function TForm1.VisibleUIElementsToInteger: integer;
var
  ui : TUIElement;
begin
  result := 0;
  for ui in [Low(TUIElement) .. High(TUIElement)] do
    if ui IN VisibleUIElements then
      result := result + 1 shl Integer(ui);
end;

end.
